@@ -319,13 +319,23 @@ process_file (const char *fname)
 	  return 0;
 	}
       else if (likely (elf_kind (elf) == ELF_K_AR))
-	return handle_ar (fd, elf, NULL, fname);
+	{
+	  int result = handle_ar (fd, elf, NULL, fname);
+
+	  if (unlikely  (close (fd) != 0))
+	    error (EXIT_FAILURE, errno, gettext ("while closing '%s'"), fname);
+
+	  return result;
+	}
 
       /* We cannot handle this type.  Close the descriptor anyway.  */
       if (unlikely (elf_end (elf) != 0))
 	INTERNAL_ERROR (fname);
     }
 
+  if (unlikely (close (fd) != 0))
+    error (EXIT_FAILURE, errno, gettext ("while closing '%s'"), fname);
+
   error (0, 0, gettext ("%s: file format not recognized"), fname);
 
   return 1;
@@ -396,9 +406,6 @@ handle_ar (int fd, Elf *elf, const char *prefix, const char *fname)
   if (unlikely (elf_end (elf) != 0))
     INTERNAL_ERROR (fname);
 
-  if (unlikely  (close (fd) != 0))
-    error (EXIT_FAILURE, errno, gettext ("while closing '%s'"), fname);
-
   return result;
 }
 
@@ -413,7 +420,7 @@ show_sysv (Elf *elf, const char *prefix, const char *fname,
 
   /* Get the section header string table index.  */
   size_t shstrndx;
-  if (unlikely (elf_getshstrndx (elf, &shstrndx) < 0))
+  if (unlikely (elf_getshdrstrndx (elf, &shstrndx) < 0))
     error (EXIT_FAILURE, 0,
 	   gettext ("cannot get section header string table index"));
 
@@ -487,7 +494,7 @@ show_sysv_one_line (Elf *elf)
 {
   /* Get the section header string table index.  */
   size_t shstrndx;
-  if (unlikely (elf_getshstrndx (elf, &shstrndx) < 0))
+  if (unlikely (elf_getshdrstrndx (elf, &shstrndx) < 0))
     error (EXIT_FAILURE, 0,
 	   gettext ("cannot get section header string table index"));
 
